import requests
from ._result import Result


def get_file_for_name(name):
    first_letter = name[0].lower()
    file_path = f'https://raw.githubusercontent.com/SermetPekin/nameisok/680f11c457858a058ad81bda819008cb341d3bd2/raw/raw-{first_letter}.txt'
    return file_path


def check_package_name_extra(name):
    file_url = get_file_for_name(name)
    response = requests.get(file_url)
    ok = name in response.text
    available_text = 'Available' if not ok else 'Not Available'
    r = Result(ok, 999, available_text)
    return r



import requests

from ._result import Result
from .c_check import check_package_name_extra
from .similar import get_existing_packages, check_name_similarity


def get_check_url(p_name: str) -> str:  # test ok
    # url = f"https://pypi.org/project/{p_name}/"
    url = f"https://pypi.org/simple/{p_name}/"
    return url


def get_request_eval_result(url: str) -> Result:
    req = requests.get(url)  # not testing this part [requests]
    return eval_req(req.status_code, requests.codes.ok)  # test ok


def eval_req(status_code: int, ok_code: int = requests.codes.ok) -> Result:  # test ok
    exists = status_code == ok_code
    available_text = 'Available' if not exists else 'Not Available'
    r = Result(exists, status_code, available_text)
    return r


def get_status_package(p_name: str) -> Result:
    url = get_check_url(p_name)  # test ok
    result = get_request_eval_result(url)  # auto testing partly ok
    if not result.exists:
        result = check_package_name_extra(p_name)
    return result


def extra_checks_if_available(p_name):
    print('Similarity check begins...')
    check_similarity(p_name)


def show_status_console_available(package_name: str) -> None:  # test ok

    template = f"\n. 🎉 Wow! `{package_name}` is available!"
    print(template)


def show_status_console_taken(package_name: str) -> None:  # test ok
    template = f"\n  ❌ `{package_name}` is already taken."
    print(template)


def show_status_similar_exists(package_name: str, similar_names: list) -> None:  # test ok
    if similar_names:
        print(
            f" ❌ Unfortunately, the name '{package_name}' is too similar to existing projects: {', '.join(similar_names)}")
    else:
        print(f"I have good news 🎉 the name '{package_name}' is available and sufficiently unique.")


def check_similarity(new_package_name):
    similar_names = check_name_similarity(new_package_name, get_existing_packages())
    return similar_names


def action_get_status_package(package_name: str) -> bool:
    r = get_status_package(package_name)
    if r.exists:  # pypi + database was checked
        show_status_console_taken(package_name)
        return not r.exists
    print(f'First check passed for `{package_name}` 🎉. Now I will check similar packages.')
    # similarity check
    similar_names = check_similarity(package_name)
    show_status_similar_exists(package_name, similar_names)
    return not similar_names.__len__() > 0


def get_status_package_cli(package_name: str, action=None) -> bool | list[bool]:  # test ok
    if action is None:
        action = action_get_status_package
    if ',' in package_name:
        results = []
        for p in package_name.split(','):
            res = action(p)
            results.append(res)
        return results
    result = action(package_name)
    return result



from fuzzywuzzy import fuzz
import requests
import os
from datetime import datetime, timedelta
from pathlib import Path

CACHE_DURATION = timedelta(days=30)  # Cache duration of one month


def get_cache_file():
    PACKAGE_NAME = "nameisok"  # Your package name

    # Determine cache directory based on the OS
    if os.name == 'posix':
        cache_dir = Path.home() / '.cache' / PACKAGE_NAME  # Linux/macOS: ~/.cache/nameisok
    elif os.name == 'nt':
        cache_dir = Path(os.getenv('LOCALAPPDATA',
                                   Path.home() / 'AppData' / 'Local')) / PACKAGE_NAME  # Windows: %LOCALAPPDATA%\nameisok
    else:
        cache_dir = Path.home() / '.cache' / PACKAGE_NAME  # Default to Linux-style cache path

    cache_dir.mkdir(parents=True, exist_ok=True)  # Create the directory if it doesn't exist
    CACHE_FILE = cache_dir / 'names_cache.txt'
    return CACHE_FILE
def get_existing_packages():
    cache_file = get_cache_file()
    if cache_file.exists():
        file_mod_time = datetime.fromtimestamp(cache_file.stat().st_mtime)
        if datetime.now() - file_mod_time < CACHE_DURATION:
            with cache_file.open('r') as f:
                return [line.strip() for line in f if line.strip()]

    url = 'https://raw.githubusercontent.com/SermetPekin/nameisok/d6048743d42fed02254a01a53c1ef563141deea9/raw/names_.txt'
    print('retrieving packages...')
    response = requests.get(url)
    response.raise_for_status()

    ns = [line.strip() for line in response.text.splitlines() if line.strip()]

    with cache_file.open('w') as f:
        f.write('\n'.join(ns))

    return ns

def check_name_similarity(new_name, existing_packages, threshold=80) -> list[str]:
    similar_names = []

    existing_packages = [str(x) for x in existing_packages]
    for name in existing_packages:
        similarity_score = fuzz.ratio(new_name.lower(), name.lower())
        if similarity_score >= threshold:
            similar_names.append(name)
    return similar_names



from nameisok.core import get_check_url, eval_req, get_status_package_cli, show_status_console_available, \
    show_status_console_taken


def test_get_url():
    url: str = get_check_url('pandas')
    assert url

    assert url.endswith('/')


def test_eval_req():
    s = eval_req(200, 200)
    assert s.exists


def test_show_status_console(capsys):
    with capsys.disabled():
        show_status_console_available('test')
        show_status_console_taken('test')


def test_get_status_package_cli(capsys):
    with capsys.disabled():
        r = get_status_package_cli('pandas,example', lambda x: print(x))
        assert tuple(r ) == ( None , None )


def test_real_get_status_package_cli(capsys):
    with capsys.disabled():
        r = get_status_package_cli('pandas')
        assert r is False
        r = get_status_package_cli('pandas_12345678')
        assert r is True



import click

from .core import get_status_package_cli
from .types_ import StrOrTuple

@click.command()
@click.argument("name")
def main(name: StrOrTuple):
    """CLI command to check PyPI package name availability."""
    get_status_package_cli(name)
    print('\n')

